////////////////////////////////////////////////////////////////////////
///File		:IconButton.cs
///Author	:Richard L. Weeks
///
/// Copyright (c) 2001 by Richard L. Weeks
/// This file is provided for instructional purposes only.  
/// No warranties.
////////////////////////////////////////////////////////////////////////

using System;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.ComponentModel.Design;
using SAMS.ToolKit.Licensing;


namespace SAMS.ToolKit.Controls {
	/// <summary>
	/// IconButton Class
	/// </summary>
	[
	System.ComponentModel.Description( "SAMS IconButton Control" ),
	System.ComponentModel.Designer( typeof( SAMS.ToolKit.Design.IconButtonDesigner ) ),
	System.ComponentModel.LicenseProvider( typeof( SAMS.ToolKit.Licensing.CustomLicenseProvider ) )
	]
	public class IconButton : System.Windows.Forms.Control {
	

		#region STATIC MEMBER FIELDS

		protected static int		EDGE_PADDING = 4;
		
		#endregion

		#region Implementation Member Fields
		
		protected ButtonState		buttonState;
		protected Icon				buttonIcon;
		protected int				iconDrawWidth;
		protected bool				mousePressed;

		private   License			license	= null;
		#endregion

		#region IconButton Properties

		[
		System.ComponentModel.Description( "The Icon to be displayed in the button" ),
		System.ComponentModel.Category( "Appearance" ),
		System.ComponentModel.DefaultValue( null ),
		System.ComponentModel.Editor( typeof( SAMS.ToolKit.Design.IconEditor ), typeof( System.Drawing.Design.UITypeEditor ) )
		]
		public Icon Icon {
			get { return buttonIcon; }
			set {
				buttonIcon = value;
				Invalidate( );
				Update( );
			}
		}
		
		#endregion


		#region Construction / Initialization
		/// <summary>
		/// Simple Constructor
		/// </summary>
		public IconButton( ) {

			//validate the license
			license = LicenseManager.Validate( typeof( SAMS.ToolKit.Controls.IconButton ), this );
			System.Diagnostics.Debug.WriteLine( license.LicenseKey );
			InitializeComponent(  );
		}

		protected override void Dispose( bool disposing ) {
			if( license != null ) {
				license.Dispose( );
				license = null;
			}
			base.Dispose( disposing );
		}
		/// <summary>
		/// Initialize the default values for the IconButton
		/// </summary>
		private void InitializeComponent( ) {
			buttonIcon		= null;
			buttonState		= ButtonState.Normal;
			mousePressed	= false;
		}
		#endregion

		#region Control Method Overrides

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		protected override void OnGotFocus( EventArgs e ) {
			Invalidate( );
			base.OnGotFocus( e );
		}

		protected override void OnLostFocus( EventArgs e ) {
			Invalidate( );
			base.OnLostFocus( e );
		}

		protected override void OnTextChanged( EventArgs e ) {
			Invalidate( );
			Update( );
			base.OnTextChanged( e );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		protected override void OnSizeChanged( EventArgs e ) {
			base.OnSizeChanged( e );
			this.Invalidate( );
			this.Update( );
		}

		/// <summary>
		/// Render the IconButton
		/// </summary>
		/// <param name="e"><see cref="PaintEventArgs"/></param>
		protected override void OnPaint( PaintEventArgs e ) {
			base.OnPaint( e );
			Draw( e.Graphics );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseDown( MouseEventArgs e ) {
			if( e.Button == MouseButtons.Left ) {
				Focus( );
				Capture			= true;
				buttonState		= ButtonState.Pushed;
				mousePressed	= true;
				Invalidate( );
				Update( );
			} else
				base.OnMouseDown( e );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseUp( MouseEventArgs e ) {

			if( mousePressed && e.Button == MouseButtons.Left ) {
				Capture			= false;
				buttonState		= ButtonState.Normal;
				Invalidate( );
				Update( );
			} else
				base.OnMouseUp( e );

			mousePressed	= false;
		}

		

		#endregion

		#region Implementation

		/// <summary>
		/// Render the IconButton
		/// </summary>
		/// <param name="g"></param>
		protected virtual void Draw( Graphics g ) {

			DrawButton( g );
			
			if( buttonIcon != null )
				DrawIcon( g );
			
			DrawText( g );

			if( base.Focused )
				DrawFocusClues( g );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="g"></param>
		protected virtual void DrawButton( Graphics g ) {
			Rectangle rcButton = new Rectangle( 0, 0, this.Width, this.Height );
			if( Focused ) 
				rcButton.Inflate(-1,-1);
				
			ControlPaint.DrawButton( g, rcButton, buttonState );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="g"></param>
		protected virtual void DrawText( Graphics g ) {
			int left	= (buttonIcon == null ? 
				IconButton.EDGE_PADDING : 
				iconDrawWidth + IconButton.EDGE_PADDING);
			int width	= Width - left;
			int top		= IconButton.EDGE_PADDING;
			int height	= Height - (2*IconButton.EDGE_PADDING);

			RectangleF layoutRect = new RectangleF( left, top, width, height );
			if( ButtonState.Pushed == buttonState )
				layoutRect.Offset( 1f, 1f );

			StringFormat fmt	= new StringFormat( );
			fmt.Alignment		= StringAlignment.Center;
			fmt.LineAlignment	= StringAlignment.Center;

			SolidBrush textBrush	= new SolidBrush( this.ForeColor );
			g.DrawString( Text, Font, textBrush, layoutRect, fmt );

			textBrush.Dispose( );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="g"></param>
		protected virtual void DrawIcon( Graphics g ) {
			System.Diagnostics.Debug.Assert( buttonIcon != null, "IconButton Icon is null" );

			int top		= ((Height/2) - (buttonIcon.Height/2));
			int height	= buttonIcon.Height;
			int width	= buttonIcon.Width;

			if( (top + height) >= Height ) {
				//Scale the image to fit in (w,h) of button
				top = IconButton.EDGE_PADDING;
				int drawHeight = Height - (2*IconButton.EDGE_PADDING);
				float scale = ((float)drawHeight / (float)height);
				width = (int)((float)width*scale);
				height = drawHeight;
			}
			Rectangle iconRect = new Rectangle( IconButton.EDGE_PADDING, top, width, height);

			if( buttonState == ButtonState.Pushed )
				iconRect.Offset(1,1);
			
			if( base.Enabled )
				g.DrawIcon( buttonIcon, iconRect );
			else {
				//Draw Image in disabled state
				Image disabledImage = buttonIcon.ToBitmap( );
				disabledImage = disabledImage.GetThumbnailImage( iconRect.Width, iconRect.Height, null, new IntPtr(0) );
				ControlPaint.DrawImageDisabled( g, disabledImage, iconRect.X, iconRect.Y, base.BackColor );
			}

			this.iconDrawWidth = iconRect.Width;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="g"></param>
		protected virtual void DrawFocusClues( Graphics g ) {
			System.Drawing.Pen p = new Pen( System.Drawing.Color.Black, 1f );
			Rectangle frameRect = new Rectangle( 0, 0, this.Width, this.Height );
			g.DrawRectangle( p, frameRect );

			p.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
			frameRect = new Rectangle( 2, 2, this.Width - 6, this.Height - 6);
			if( buttonState == ButtonState.Pushed )
				frameRect.Offset( 1, 1 );
			g.DrawRectangle( p, frameRect );
			
			p.Dispose( );
		}
		#endregion

	}
}
