using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace SAMS.ToolKit.Controls
{


	public enum ListViewItemState {
		Normal,
		Hover,
		Pushed
	}
	



	[ToolboxItem( false )]
	public class ImageListViewItem : System.ComponentModel.Component {
	
		private static int EDGE_PADDING = 4;
		private static int TEXT_PADDING = 12;
		private static int LARGE		= 32;
		private static int SMALL		= 16;

		private string			text;
		private	int				imageIndex;
		private	Rectangle		imageRect;
		private	Rectangle		textRect;
		private	bool			drawLarge;
		private	float			fontHeight;
		private	Image			image;
		private	int				fontLines;
		private int				height;
		private Point			location;

		internal ImageListView	parent;

		[
		Description( "Text displayed for the current item" ),
		Category( "Appearance" )
		]
		public string Text {
			get { return text; }
			set { 
				text = value;
			}
		}

		[
		Description( "Image Index used to display associated image" ),
		Category( "Appearance" )
		]
		public int ImageIndex {
			get { return imageIndex; }
			set {
				imageIndex = value; 
			}
		}

		[
		Browsable( false ),
		EditorBrowsable( EditorBrowsableState.Advanced )
		]
		public ImageListView Parent {
			get { return parent; }
		}

		[Browsable( false )]
		public int Height {
			get { return height; }
		}

		[Browsable( false )]
		public Point Location {
			get { return location; }
		}

		public ImageListViewItem( ) {
			parent		= null;
			drawLarge	= true;
			imageIndex	= -1;
			imageRect	= new Rectangle(0,0,0,0);
			textRect	= new Rectangle(0,0,0,0);
			image		= null;
			height		= 0;
		}

		public bool HitTest( Point pt ) {
			return (imageRect.Contains( pt ) || textRect.Contains( pt ));
		}


		public int CalcHeight( Graphics g, Font font, int width, bool LargeImage ) {
			
			drawLarge = LargeImage;

			height = ( LargeImage ? LARGE : SMALL );
			
			//calc the height of the Text
			SizeF fontSize = g.MeasureString( text, font );
			fontHeight = fontSize.Height;
			fontLines = 1;
			if( LargeImage ) {
				height += TEXT_PADDING;
				if( fontSize.Width > width ) {
					//Need to wrap the text
					height += (int)((float)2*fontSize.Height);
					fontLines = 2;
				} else
					height += (int)fontSize.Height;
			} 
			return height;
		}


		public void Draw( Graphics g, Font font, Brush foreBrush, Point location, int maxWidth, ListViewItemState state ) {
			
			if( height == 0 || imageIndex == -1) return;
			this.location = location;
			image = ( drawLarge ? parent.LargeImageList.Images[ imageIndex ] : parent.SmallImageList.Images[ imageIndex ] );
			if( drawLarge )
				DrawLarge( g, font, foreBrush, location, maxWidth, state );
			else
				DrawSmall( g, font, foreBrush, location, maxWidth, state );
		}

		public void DrawSmall( Graphics g, Font font, Brush foreBrush, Point location, int maxWidth, ListViewItemState state  ) {

			imageRect = new Rectangle( location.X + EDGE_PADDING, location.Y, SMALL, SMALL );
			textRect = new Rectangle( location.X + (2*EDGE_PADDING) + imageRect.Width, location.Y, maxWidth, (int)fontHeight );
			
			StringFormat fmt	= new StringFormat( );
			fmt.Alignment		= StringAlignment.Near;
			fmt.LineAlignment	= StringAlignment.Center;
			fmt.Trimming		= StringTrimming.EllipsisCharacter;
			fmt.FormatFlags		= StringFormatFlags.NoWrap;
			
			g.DrawString( text, font, foreBrush, textRect, fmt );
			
			DrawImage( g, imageRect, state );
		}

		public void DrawLarge( Graphics g, Font font, Brush foreBrush, Point location, int maxWidth, ListViewItemState state  ) {

			imageRect = new Rectangle( (location.X + (maxWidth/2) - (LARGE/2)), location.Y, LARGE, LARGE );
			textRect = new Rectangle( location.X + EDGE_PADDING, imageRect.Bottom + TEXT_PADDING, maxWidth, fontLines*(int)fontHeight );
			
			StringFormat fmt	= new StringFormat( );
			fmt.Alignment		= StringAlignment.Center;
			fmt.LineAlignment	= StringAlignment.Center;
			fmt.Trimming		= StringTrimming.EllipsisCharacter;
			
			g.DrawString( text, font, foreBrush, textRect, fmt );

			DrawImage( g, imageRect, state );

		}

		private void DrawImage( Graphics g, Rectangle destRect, ListViewItemState state ) {
			Rectangle drawRect = destRect;
			switch( state ) {
				case ListViewItemState.Normal:
					g.DrawImage( image, drawRect );
					break;

				case ListViewItemState.Pushed:
					drawRect.Inflate(-2,-2);
					ControlPaint.DrawBorder3D( g, drawRect, Border3DStyle.Adjust | Border3DStyle.SunkenInner, Border3DSide.All );
					drawRect.Inflate(2,2);
					drawRect.X +=1;drawRect.Width-=2;
					drawRect.Y +=1;drawRect.Height-=2;
					g.DrawImage( image, drawRect );
					
					break;

				case ListViewItemState.Hover:
					drawRect.Inflate(-2,-2);
					ControlPaint.DrawBorder3D( g, drawRect, Border3DStyle.Adjust | Border3DStyle.RaisedOuter, Border3DSide.All );
					drawRect.X -=1;drawRect.Width+=2;
					drawRect.Y -=1;drawRect.Height+=2;
					g.DrawImage( image, drawRect );
					break;
			}
		}
	}
}
