using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections;

namespace SAMS.ToolKit.Controls
{

	public enum HitTestType {
		TABS,
		CLIENT
	}

	#region OutlookBar EventArgs

	public class OutlookBarSelectedTabChangedEventArgs : EventArgs {
		private OutlookBarTab		activeTab;
		private OutlookBarTab		prevTab;
		

		public OutlookBarTab ActiveTab {
			get { return activeTab; }
		}

		public OutlookBarTab PreviousTab {
			get { return prevTab; }
		}

		public OutlookBarSelectedTabChangedEventArgs( OutlookBarTab active, OutlookBarTab prev ) {
			activeTab	= active;
			prevTab		= prev;
		}
	}

	public delegate void OutlookBarSelectedTabChangedEventHandler( object sender, OutlookBarSelectedTabChangedEventArgs e );
	#endregion

	#region OutlookBar Control

	[
	Description( "The OutlookBar Control aka Shortcut Bar" ),
	DefaultEvent( "SelectedTabChanged" ),
	DefaultProperty( "Tabs" ),
	Designer( typeof( SAMS.ToolKit.Design.OutlookBarDesigner ) )
	]
	public class OutlookBar : System.Windows.Forms.ContainerControl {

		

		#region Instance Members, Properties and Events

		private System.ComponentModel.Container		components	= null;
		private TabCollection						tabCollection;
		private int									activeTabIndex;
		private int									hitTabIndex;
		private Rectangle							activeClientRect;
		private int									tabHeight;
		

		[
		Description( "Raised when the active outlooktab changes" ),
		Category( "Behavior" )
		]
		public event OutlookBarSelectedTabChangedEventHandler SelectedTabChanged;


		[
		Description( "The collection of OutlookTabs" ),
		Category( "Appearance" ),
		DesignerSerializationVisibility( DesignerSerializationVisibility.Content )
		]
		public TabCollection Tabs {
			get { return tabCollection; }
			set {
				tabCollection.Clear( );
				tabCollection = value;
			}
		}

		[ 
		Browsable( false ),
		DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden )
		]
		public int ActiveTabIndex {
			get {
				return activeTabIndex;
			}
			set {
				ActivateTab( (int)value );
			}
		}

		[ 
		Browsable( false ),
		DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden )
		]
		public OutlookBarTab ActiveTab {
			get { return tabCollection[ activeTabIndex ]; }
		}

		[ DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden ) ]
		public new ControlCollection Controls {
			get { return base.Controls; }
		}


		#endregion

		#region Construction, Initialization and Public Instance Methods

		public OutlookBar( ) {
			InitializeComponent( );

			//Set default values
			tabCollection		= new TabCollection( this );
			activeTabIndex		= -1;
			hitTabIndex			= -1;
			activeClientRect	= new Rectangle(0,0,0,0);
			tabHeight			= 24;
		}

		//Required for Designer Support
		private void InitializeComponent( ) {
		}

		public bool HitTest( HitTestType testType, Point pt, out int TabIndex ) {
			bool bResult = false;
			TabIndex = -1;

			if( testType == HitTestType.CLIENT ) {
				bResult = this.activeClientRect.Contains( pt );
			} else {
				int hitTabCache = this.hitTabIndex;
				bResult = HitTestTabs( pt );
				if( bResult )
					TabIndex = hitTabIndex;
				hitTabIndex = hitTabCache;
			}
			return bResult;
		}
		
		#endregion

		#region ContainerControl Overrides

		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		protected override void OnFontChanged( EventArgs e ) {
			base.OnFontChanged( e );
			ReCalctabHeight( );
			Invalidate( );
		}

		protected override void OnSizeChanged( EventArgs e ) {
			base.OnSizeChanged( e );
			ReCalcClientRect( );
			Invalidate( );
		}

		protected override void OnMouseDown( MouseEventArgs e ) {
			if( e.Button == MouseButtons.Left ) {
				if( HitTestTabs( new Point( e.X, e.Y ) ) ) {
					Graphics g		= CreateGraphics( );
					DrawTab( g, hitTabIndex, ButtonState.Pushed );
					g.Dispose( );
				} else
					base.OnMouseDown( e );
			} else 
				base.OnMouseDown( e );
		}

		protected override void OnMouseUp( MouseEventArgs e ) {
			if( (e.Button == MouseButtons.Left) && (hitTabIndex != -1) ) {
				
				Graphics g = CreateGraphics( );
				DrawTab( g, hitTabIndex, ButtonState.Normal );
				g.Dispose( );

				if( tabCollection[ hitTabIndex ].HitTest( new Point( e.X, e.Y ) ) ) {
					int activeIndex = hitTabIndex;
					hitTabIndex = -1;
					ActivateTab( activeIndex );
				}
				
			} else
				base.OnMouseUp( e );
		}

		protected override void OnPaint( PaintEventArgs e ) {
			base.OnPaint( e );
			DrawBorder( e.Graphics );
			DrawTabs( e.Graphics );
			ActivateChildControl( );
		}
		#endregion

		#region Protected/Internal/Private Instance Methods
		
		protected virtual bool HitTestTabs( Point pt ) {

			for( int tabIndex = 0; tabIndex < tabCollection.Count; tabIndex++ ) {
				if( tabCollection[ tabIndex ].HitTest( pt ) ) {
					hitTabIndex = tabIndex;
					return true;
				}
			}
			hitTabIndex = -1;
			return false;
		}

		protected virtual void ActivateTab( int tabIndex ) {

			if( (tabIndex == activeTabIndex) || (tabIndex >= tabCollection.Count))
				return;

			OutlookBarTab prevTab	= tabCollection[ activeTabIndex ];
			OutlookBarTab activeTab = tabCollection[ tabIndex ];

			DeActivateTab( activeTabIndex );
			
			activeTabIndex = tabIndex;
			
			ReCalcClientRect( );

			Invalidate( );
			Update( );
			ActivateChildControl( );

			OnSelectedTabIndexChanged( new OutlookBarSelectedTabChangedEventArgs( activeTab, prevTab ) );
		}

		protected virtual void DeActivateTab( int tabIndex ) {
			OutlookBarTab T = tabCollection[ tabIndex ];
			if( (T != null) && (T.Child != null) )
				T.Child.Visible = false;
		}

		protected virtual void ActivateChildControl( ) {
			OutlookBarTab T = tabCollection[ activeTabIndex ];
			if( (T != null ) && (T.Child != null) ) {
				T.Child.Visible		= true;
				T.Child.Size		= new Size( activeClientRect.Width, activeClientRect.Height );
				T.Child.Location	= new Point( activeClientRect.Left, activeClientRect.Top );
				T.Child.BringToFront( );
			}
		}

		protected virtual void ReCalctabHeight( ) {
			Graphics g = CreateGraphics( );
			tabHeight =	(int)(Font.GetHeight( g ) + (float)(2*OutlookBarTab.EDGE_PADDING));
			g.Dispose( );
		}

		protected virtual void ReCalcClientRect( ) {
			activeClientRect = new Rectangle( 1, 1, Width - 2, Height - 2 );
			activeClientRect.Y += tabHeight * ( activeTabIndex + 1 );
			activeClientRect.Height -= tabCollection.Count * tabHeight;
		}

		protected virtual void DrawBorder( Graphics g ) {
			ControlPaint.DrawBorder3D( g, new Rectangle(0,0,Width,Height), Border3DStyle.Etched, Border3DSide.All );
		}

		protected virtual void DrawTabs( Graphics g ) {

			if( tabCollection.Count == 0 )
				return;

			Rectangle tabRect = new Rectangle( 1, 1, Width - 2, tabHeight );
			for( int tabIndex = 0; tabIndex < tabCollection.Count; tabIndex++ ) {
				OutlookBarTab T = tabCollection[ tabIndex ];
				DrawTab( g, T, tabRect, (tabIndex == hitTabIndex ? ButtonState.Pushed : ButtonState.Normal) );

				if( tabIndex == activeTabIndex )
					tabRect.Y = activeClientRect.Bottom;
				else
					tabRect.Y += tabHeight;
			}
		}

		protected virtual void DrawTab( Graphics g, int tabIndex ) {
			OutlookBarTab T = tabCollection[ tabIndex ];
			DrawTab( g, T, T.TabRect, T.ButtonState );
		}

		protected virtual void DrawTab( Graphics g, int tabIndex, ButtonState buttonState ) {
			OutlookBarTab T = tabCollection[ tabIndex ];
			DrawTab( g, T, T.TabRect, buttonState );
		}
		protected virtual void DrawTab( Graphics g, int tabIndex, Rectangle tabRect, ButtonState buttonState ) {
			DrawTab( g, tabCollection[ tabIndex ], tabRect, buttonState );
		}

		protected virtual void DrawTab( Graphics g, OutlookBarTab outlookTab, Rectangle tabRect, ButtonState buttonState ) {
			outlookTab.Draw( g, tabRect, Font, buttonState );
		}

		protected virtual void SubscribeToTabEvents( OutlookBarTab tab ) {
			System.Reflection.EventInfo[] events = tab.GetType( ).GetEvents(  );

			foreach( System.Reflection.EventInfo ei in events )
				ei.AddEventHandler( tab, new EventHandler( this.OnTabPropertyChanged ) );
		}

		protected virtual void UnSubscribeToTabEvents( OutlookBarTab tab ) {
			System.Reflection.EventInfo[] events = tab.GetType( ).GetEvents( );

			foreach( System.Reflection.EventInfo ei in events )
				ei.RemoveEventHandler( tab, new EventHandler( this.OnTabPropertyChanged ) );
		}

		protected virtual void OnTabPropertyChanged( object sender, EventArgs e ) {
			Control ctrl = ((OutlookBarTab)sender).Child;
			if(  ctrl != null && !Controls.Contains( ctrl ) ) {
				ctrl.Visible = false;
				this.Controls.Add( ctrl );
			}
			Invalidate( );
		}

		protected virtual void OnSelectedTabIndexChanged( OutlookBarSelectedTabChangedEventArgs e ) {
			if( SelectedTabChanged != null )
				SelectedTabChanged( this, e );
		}

		#endregion


		

		#region TabCollection
		public class TabCollection : ICollection, IEnumerable, IList {

			private OutlookBar		owner				= null;
			private ArrayList		internalArrayList	= null;


			public TabCollection( OutlookBar parent ) {
				owner = parent;
				internalArrayList = new ArrayList( );
			}

			//ICollection Interface implementation
			public int Count {
				get { return internalArrayList.Count; }
			}
			public bool IsSynchronized {
				get { return internalArrayList.IsSynchronized; }
			}
			public object SyncRoot {
				get { return internalArrayList.SyncRoot; }
			}

			public void CopyTo( Array array, int arrayIndex ) {
				internalArrayList.CopyTo( array, arrayIndex );
			}

			//IList interface implementation
			public bool IsFixedSize {
				get { return false; }
			}

			public bool IsReadOnly {
				get { return false; }
			}

			//Only used when Interface IList is acquired.
			//IList iList = (IList)TabCollection;
			object IList.this[ int index ] {
				get { return this[ index ]; }
				set { this[ index ] = (OutlookBarTab)value; }
			}

			int IList.Add( object o ) {
				return this.Add( (OutlookBarTab)o );
			}

			void IList.Clear( ) {
				this.Clear( );
			}

			public bool Contains( object o ) {
				return internalArrayList.Contains( o );
			}

			public int IndexOf( object o ) {
				return internalArrayList.IndexOf( o );
			}

			void IList.Insert( int index, object o ) {
				this.Insert( index, (OutlookBarTab)o );
			}

			void IList.Remove( object o ) {
				this.Remove( (OutlookBarTab)o );
			}

			void IList.RemoveAt( int index ) {
				this.RemoveAt( index );
			}

			//IEnumerable interface implementation
			public IEnumerator GetEnumerator( ) {
				return internalArrayList.GetEnumerator( );
			}

			//TabCollection implementation
			public int Add( OutlookBarTab tab ) {
				owner.SubscribeToTabEvents( tab );
				int idx = internalArrayList.Add( tab );
				if( tab.Child != null )
					owner.Controls.Add( tab.Child );
				owner.ActivateTab( idx );
				return idx;
			}

			public void AddRange( OutlookBarTab[] tabs ) {
				foreach( OutlookBarTab T in tabs )
					Add( T );
			}

			public void Remove( OutlookBarTab tab ) {
				RemoveAt( IndexOf( tab ) );
			}

			public void RemoveAt( int index ) {
				OutlookBarTab T = this[index];
				owner.Controls.Remove( T.Child );
				internalArrayList.Remove( T );
				owner.UnSubscribeToTabEvents( T );
				
				if( internalArrayList.Count > 0 ) {
					//Was this the active Tab?
					if( index == owner.activeTabIndex ) {
						index++; index %=( Count+1 );
					}
				}
				owner.Invalidate( );
			}

			public void Insert( int index, OutlookBarTab tab ) {
				internalArrayList.Insert( index, tab );
				if( tab.Child != null )
					owner.Controls.Add( tab.Child );
				owner.SubscribeToTabEvents( tab );
				//is the new tab before or after the current tab?
				if( index <= owner.activeTabIndex )
					owner.activeTabIndex++;
				owner.Invalidate( );
			}

			public void Clear( ) {
				foreach( OutlookBarTab tab in internalArrayList )
					owner.UnSubscribeToTabEvents( tab );

				owner.Controls.Clear( );
				internalArrayList.Clear( );
				owner.activeTabIndex = -1;
				owner.Invalidate( );
			}

			public OutlookBarTab this[ int index ] {
				get { 
					// The CollectionEditor is famous for passing the index value of
					// -1.  For what reason, I have no idea.
					try {
						return (OutlookBarTab)internalArrayList[index]; 
					} catch( Exception e ) {
						System.Diagnostics.Debug.WriteLine( e.Message );
						System.Diagnostics.Debug.WriteLine( string.Format( "TabCollection[{0}] invalid index", index ) );
					} 
					return null;
				}
				set { 
					internalArrayList[index] = value; 
				}
			}
		}
		#endregion

	}

	#endregion
	
}
