using System;
using System.ComponentModel;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;


namespace SAMS.ToolKit.Controls {

	public class ImageListViewEventArgs : EventArgs {
		private int	itemIndex;

		public int ItemIndex { get { return itemIndex; } }

		public ImageListViewEventArgs( int index ) {
			itemIndex = index;
		}
	}

	public delegate void ItemClickedEventHandler( object sender, ImageListViewEventArgs e );

	[
	Description( "ImageListView Control" ),
	DefaultEvent( "ItemClicked" ),
	DefaultProperty( "Items" ),
	Designer( typeof( SAMS.ToolKit.Design.ItemListViewDesigner ) )
	]
	public class ImageListView : Control {
		
		private int				topIndex			= 0;
		private int				lastIndex			= 0;
		private Point			firstItemLocation	= new Point(0,0);
		private Point			lastItemLocation	= new Point(0,0);
		private bool			bScrollUp			= false;
		private bool			bScrollDown			= false;
		private bool			bLargeIcons			= true;
		private int				hitIndex			= 0;
		private int				hoverIndex			= -1;
		private ImageList		largeImageList		= null;
		private ImageList		smallImageList		= null;
		private ScrollButton	downScroll;
		private ScrollButton	upScroll;
		private ListViewItemCollection	items;



		[
		Description( "Event raised when an ImageItem is clicked" ),
		Category( "Behavior" )
		]
		public event ItemClickedEventHandler ItemClicked;

		[
		Description( "Large Image List" ),
		Category( "Appearance" )
		]
		public ImageList LargeImageList {
			get { return largeImageList; }
			set { largeImageList = value; Invalidate( );}
		}

		[
		Description( "Small Image List" ),
		Category( "Appearance" )
		]
		public ImageList SmallImageList {
			get { return smallImageList; }
			set { smallImageList = value; Invalidate( ); }
		}

		
		[
		Description( "Image Items Collection" ),
		Category( "Appearance" ),
		DesignerSerializationVisibility( DesignerSerializationVisibility.Content )
		]
		public ListViewItemCollection Items {
			get { return items; }
			set { items = value; }
		}

		[
		Description( "use large images" ),
		Category( "Appearance" )
		]
		public bool LargeIcons {
			get {
				return bLargeIcons;
			}
			set {
				bLargeIcons = value; 
				Recalc( );
				Invalidate();
			}
		}



		public ImageListView( ) {
			items = new ListViewItemCollection( this );
			downScroll = new ScrollButton( ScrollButtonOrientation.Down );
			upScroll = new ScrollButton( ScrollButtonOrientation.Up );
		}

		

		protected override void OnPaint( PaintEventArgs e ) {
			base.OnPaint( e );

			if( items.Count == 0 ) return;
			if( this.bLargeIcons ) {
				if( this.largeImageList == null ) return;
			} else {
				if( this.smallImageList == null ) return;
			}

			int ClientHeight = this.ClientRectangle.Height;
			lastIndex = topIndex;
			int heightAdjust;
			Point pt = new Point(0,0);
			Brush brush = new SolidBrush( this.ForeColor );

			do {
				ImageListViewItem Item = items[ lastIndex++ ];
				heightAdjust = Item.Height;

				DrawItem( e.Graphics, brush, Item, pt, SAMS.ToolKit.Controls.ListViewItemState.Normal );

				pt.Y += heightAdjust + 25;

			} while( pt.Y < ClientHeight && (lastIndex < items.Count));

			lastIndex--;
			//Scrolling?
			
			this.bScrollUp = topIndex> 0 ? true : false;
			this.bScrollDown = ((lastIndex+1 < items.Count) || (ClientHeight < pt.Y)) ? true : false;
			
			DrawScrollButtons( e.Graphics );

		}

		protected override void OnSizeChanged( EventArgs e ) {
			base.OnSizeChanged( e );
			Graphics g = CreateGraphics( );

			foreach( ImageListViewItem T in items ) {
				T.CalcHeight( g, Font, this.ClientRectangle.Width, bLargeIcons );
			}
			this.Invalidate( );
		}

		protected override void OnMouseMove( MouseEventArgs e ) {
			
			base.OnMouseMove( e );

			Point pt = new Point( e.X, e.Y );
			Graphics g = CreateGraphics( );
			Brush brush = new SolidBrush( this.ForeColor );
			bool bHaveHover = false;

			if( hoverIndex != -1 ) {
				ImageListViewItem Item = items[ hoverIndex ];
				if( !Item.HitTest( pt ) )
					Item.Draw( g, Font, brush, Item.Location, this.ClientRectangle.Width, ListViewItemState.Normal );
				else {
					g.Dispose( );
					brush.Dispose( );
					return;
				}
			}
			
			for( int index = this.topIndex; index <= this.lastIndex; index++ ) {
				ImageListViewItem Item = items[ index ];
				if( Item.HitTest( pt ) ) {
					Item.Draw( g, Font, brush, Item.Location, this.ClientRectangle.Width, ListViewItemState.Hover );
					hoverIndex = index;
					bHaveHover = true;
					break;
				}
			}
			
			if( !bHaveHover )
				hoverIndex = -1;

			g.Dispose( );
			brush.Dispose( );
		}

		protected override void OnMouseDown( MouseEventArgs e ) {

			if( e.Button == MouseButtons.Left ) {
				Point pt = new Point( e.X, e.Y );
				Graphics g = CreateGraphics( );

				//Hit test for scrolling
				if( this.bScrollUp )
					if( this.upScroll.HitTest( pt ) ) {
						upScroll.Draw( g, ButtonState.Pushed );
						g.Dispose( );
						return;
					}
		
				if( this.bScrollDown )
					if( this.downScroll.HitTest( pt ) ) {
						downScroll.Draw( g, ButtonState.Pushed );
						g.Dispose( );
						return;
					}

				//Hit Test items
				for( hitIndex = topIndex; hitIndex <= lastIndex; hitIndex++ ) {
					ImageListViewItem Item = items[ hitIndex ];
					if( Item.HitTest( pt ) ) {
						Brush brush = new SolidBrush( this.ForeColor );
						Item.Draw( g, Font, brush, Item.Location, this.ClientRectangle.Width, ListViewItemState.Pushed );
						brush.Dispose( );
						g.Dispose( );
						return;
					}
				}
			}
			hitIndex = -1;
			base.OnMouseDown( e );
		}

		protected override void OnMouseUp( MouseEventArgs e ) {

			if( e.Button == MouseButtons.Left ) {
				Point pt = new Point( e.X, e.Y );
				Graphics g = CreateGraphics( );
			
				//Hit test for scrolling
				if( this.bScrollUp ) {
					upScroll.Draw( g, ButtonState.Normal );
					if( this.upScroll.HitTest( pt ) ) {
						g.Dispose( );
						this.topIndex--;
						this.Invalidate( );
						return;
					}
				}
		
				if( this.bScrollDown ) {
					downScroll.Draw( g, ButtonState.Normal );
					if( this.downScroll.HitTest( pt ) ) {
						g.Dispose( );
						this.topIndex++;
						this.Invalidate( );
						return;
					}
				}

				//Test Image Items
				if( hitIndex != -1 ) {
					ImageListViewItem Item = items[ hitIndex ];
					Brush brush = new SolidBrush( this.ForeColor );
					Item.Draw( g, Font, brush, Item.Location, this.ClientRectangle.Width, ListViewItemState.Normal );
					brush.Dispose( );
					g.Dispose( );
					if( Item.HitTest( pt ) )
						this.OnItemClicked( new ImageListViewEventArgs( hitIndex ) );
					hitIndex = -1;
				}
			}
			base.OnMouseUp( e );
		}

		
		protected void DrawScrollButtons( Graphics g ) {
			if( bScrollUp ) {
				upScroll.Location = new Point( this.ClientRectangle.Width - 20, 20 );
				upScroll.Draw( g, ButtonState.Normal );
			}

			if( bScrollDown ) {
				downScroll.Location = new Point( this.ClientRectangle.Width - 20, this.ClientRectangle.Bottom - 20 );
				downScroll.Draw( g, ButtonState.Normal );
			}
		}

		protected void DrawItem( Graphics g, Brush brush, ImageListViewItem Item, Point pt, ListViewItemState state ) {
			Item.Draw( g, Font, brush, pt, this.ClientRectangle.Width, state );
		}

		protected void Recalc( ) {
			Graphics g = CreateGraphics( );
			foreach( ImageListViewItem T in items )
				T.CalcHeight( g, Font, this.ClientRectangle.Width, bLargeIcons );
		
			g.Dispose( );
		}


		protected virtual void OnItemClicked( ImageListViewEventArgs e ) {
			if( ItemClicked != null )
				ItemClicked( this, e );
		}

		public class ListViewItemCollection : ICollection, IEnumerable, IList {

			private System.Collections.ArrayList	internalArrayList;
			private ImageListView					owner;


			public ListViewItemCollection( ImageListView parent ) {
				owner = parent;
				internalArrayList = new ArrayList( );
			}


			//ICollection Interface Implementation
			public int Count {
				get { return internalArrayList.Count; }
			}

			public bool IsSynchronized {
				get { return internalArrayList.IsSynchronized; }
			}

			public object SyncRoot {
				get { return internalArrayList.SyncRoot; }
			}

			public void CopyTo( Array array, int arrayIndex ) {
				internalArrayList.CopyTo( array, arrayIndex );
			}


			//IEnumerable Interface Implementation
			public IEnumerator GetEnumerator( ) {
				return internalArrayList.GetEnumerator( );
			}

			//IList Interface Implementation
			public bool IsFixedSize {
				get { return internalArrayList.IsFixedSize; }
			}
			public bool IsReadOnly {
				get { return internalArrayList.IsReadOnly; }
			}

			object IList.this[ int index ] {
				get { return this[ index ]; }
				set { this[ index ] = (ImageListViewItem)value; }
			}

			int IList.Add( object o ) {
				return this.Add( (ImageListViewItem)o );
			}

			void IList.Clear( ) {
				this.Clear( );
			}

			public bool Contains( object o ) {
				return internalArrayList.Contains( o );
			}

			public int IndexOf( object o ) {
				return internalArrayList.IndexOf( o );
			}

			
			void IList.Insert( int index, object o ) {
				this.Insert( index, (ImageListViewItem)o );
			}

			void IList.Remove( object o ) {
				this.Remove( (ImageListViewItem)o );
			}

			void IList.RemoveAt( int index ) {
				this.RemoveAt( index );
			}

			//ListViewItemCollection Implementation
			public int Add( ImageListViewItem ilvi ) {
				int index = internalArrayList.Count;
				Insert( internalArrayList.Count, ilvi );
				return index;
			}

			public void AddRange( ImageListViewItem[] ilvi ) {
				foreach( ImageListViewItem T in ilvi )
					Add( T );
			}

			public void Remove( ImageListViewItem ilvi  ) {
				RemoveAt( IndexOf( ilvi ) );
			}

			public void RemoveAt( int index ) {
				internalArrayList.RemoveAt( index );
				owner.Invalidate( );
			}

			public void Insert( int index, ImageListViewItem ilvi ) {
				internalArrayList.Insert( index, ilvi );
				ilvi.parent = owner;
				ilvi.CalcHeight( owner.CreateGraphics( ), owner.Font, owner.ClientRectangle.Width, owner.bLargeIcons );
				owner.Invalidate( );
				owner.Update( );
			}

			public void Clear( ) {
				internalArrayList.Clear( );
				owner.Invalidate( );
			}

			public ImageListViewItem this[ int index ] {
				get {
					try {
						return (ImageListViewItem)internalArrayList[ index ];
					} catch( Exception ) {
						return null;
					}
				}
				set {
					try {
						internalArrayList[ index ] = value;
						owner.Invalidate( );
					} catch( Exception ) { }
				}
			}
		}
	}
}
