using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace SAMS.ToolKit.Controls
{
	
	[
	Description( "OutlookBarTab Component" ),
	ToolboxItem( false )
	]
	public class OutlookBarTab	: System.ComponentModel.Component {
	
		#region STATIC FIELDS
		internal static int		EDGE_PADDING = 4;
		#endregion

		#region Protected Instance Members
		protected string			text;
		protected StringAlignment	textAlignment;
		protected Color				foreColor;
		protected Icon				tabIcon;
		protected Control			child;
		
		protected Rectangle			tabRect;
		protected Rectangle			iconRect;
		protected ButtonState		buttonState;
		#endregion
		

		#region Internal Events

		[Description( "Event rasied when the Text property has been modified" )]
		public event EventHandler	TextChanged;
		
		[Description( "Event rasied when the TextAlignment property has been modified" )]
		public event EventHandler	TextAlignmentChanged;
		
		[Description( "Event rasied when the ForeColor property has been modified" )]
		public event EventHandler	ForeColorChanged;
		
		[Description( "Event rasied when the Icon property has been modified" )]
		public event EventHandler	IconChanged;

		[Description( "Event rasied when the Child property has been modified" )]
		public event EventHandler	ChildChanged;

		#endregion


		#region Properties

		[
		Description( "The text displayed on the control tab" ),
		Category( "Appearance" )
		]
		public string Text {
			get { return text; }
			set {
				if( !text.Equals( (string)value ) ) {
					text = value;
					OnTextChanged( new EventArgs( ) );
				}
			}
		}

		[
		Description( "Specifies the alignment of the text within the control tab" ),
		Category( "Appearance" )
		]
		public StringAlignment Alignment {
			get { return textAlignment; }
			set {
				if( textAlignment != value ) {
					textAlignment = value;
					OnTextAlignmentChanged( new EventArgs( ) );
				}
			}
		}

		[
		Description( "Specifies the color used to render the text on the tab control" ),
		Category( "Appearance" )
		]
		public Color ForeColor {
			get { return foreColor; }
			set {
				if( foreColor != value ) {
					foreColor = value;
					OnForeColorChanged( new EventArgs( ) );
				}
			}
		}

		[
		Description( "Specifies the Icon to drawn on the control tab" ),
		Category( "Appearance" ),
		DefaultValue( null )
		]
		public Icon Icon {
			get { return tabIcon; }
			set {
				if( tabIcon != value ) {
					tabIcon = value;
					OnIconChanged( new EventArgs( ) );
				}
			}
		}

		[
		Description( "Child control hosted within the control tab" ),
		Category( "Behavior" )
		]
		public Control Child {
			get { return child; }
			set {
				if( child != value ) {
					child = value;
					OnChildChanged( new EventArgs( ) );
				}
			}
		}

		[ Browsable( false ) ]
		internal ButtonState ButtonState {
			get { return buttonState; }
		}

		[ Browsable( false ) ]
		internal Rectangle TabRect {
			get { return tabRect; }
		}
		
		#endregion

		public OutlookBarTab()	
		{
			text = "";
			foreColor = System.Drawing.SystemColors.ControlText;
			child = null;
			tabIcon = null;
			textAlignment = StringAlignment.Center;
		}

		public virtual void Draw( Graphics g, 
			                      Rectangle destRect, 
			                      Font font, 
			                      ButtonState buttonState ) {
			
			if( destRect.IsEmpty ) return;

			tabRect = destRect;
			this.buttonState = buttonState;

			DrawButton( g );
			if( tabIcon != null )
				DrawIcon( g );
			DrawText( g, font );
		}

		public bool HitTest( Point pt ) {
			return tabRect.Contains( pt );
		}
		
		#region Protected Instance Methods


		
		protected virtual void DrawButton( Graphics g ) {
			ControlPaint.DrawButton( g, tabRect, buttonState );
		}

		
		protected virtual void DrawIcon( Graphics g ) {
			int top = tabRect.Top + EDGE_PADDING;
			int left = tabRect.Left + EDGE_PADDING;
			int width = tabIcon.Width;
			int height = tabIcon.Height;

			//Does the Icon need scaled?
			if( (top+height) >= (tabRect.Height - (2*EDGE_PADDING))) {
				float maxHeight = (tabRect.Height - (2*EDGE_PADDING));
				float scaleFactor = maxHeight / (float)height;
				height = (int)((float)height*scaleFactor);
				width = (int)((float)width*scaleFactor);
			}

			iconRect = new Rectangle( left, top, width, height );
			if( buttonState == ButtonState.Pushed )
				iconRect.Offset(1,1);

			g.DrawIcon( tabIcon, iconRect );
		}

		protected virtual void DrawText( Graphics g, Font font ) {
			Rectangle textRect = tabRect;

			textRect.X += EDGE_PADDING;
			textRect.Width -= EDGE_PADDING;
			textRect.Y += EDGE_PADDING;
			textRect.Height -= EDGE_PADDING;

			//Adjust for possible icon
			if( tabIcon != null ) {
				textRect.X += iconRect.Width;
				textRect.Width -= iconRect.Width;
			}

			//Adjust for button state
			if( buttonState == ButtonState.Pushed )
				textRect.Offset(1,1);

			//Render the Text
			StringFormat fmt	= new StringFormat( );
			fmt.Alignment		= textAlignment;
			fmt.LineAlignment	= StringAlignment.Center;
			fmt.Trimming		= StringTrimming.EllipsisCharacter;
			fmt.FormatFlags		= StringFormatFlags.NoWrap | StringFormatFlags.LineLimit;

			Brush textBrush = new SolidBrush( foreColor );

			g.DrawString( text, font, textBrush, textRect, fmt );

			textBrush.Dispose( );
		}

		
		protected virtual void OnTextChanged( EventArgs e ) {
			if( TextChanged != null )
				TextChanged( this, e );
		}
		protected virtual void OnTextAlignmentChanged( EventArgs e ) {
			if( TextAlignmentChanged != null )
				TextAlignmentChanged( this, e );
		}
		protected virtual void OnForeColorChanged( EventArgs e ) {
			if( ForeColorChanged != null )
				ForeColorChanged( this, e );
		}
		protected virtual void OnIconChanged( EventArgs e ) {
			if( IconChanged != null )
				IconChanged( this, e );
		}
		protected virtual void OnChildChanged( EventArgs e ) {
			if( ChildChanged != null )
				ChildChanged( this, e );
		}
		#endregion
	}
}
