using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using SAMS.ToolKit.Controls;

namespace SAMS.ToolKit.Design
{
	
	public class OutlookBarDesigner : System.Windows.Forms.Design.ParentControlDesigner {

		private IDesignerHost		designerHost		 = null;
		private IToolboxService		toolboxService		 = null;
		private ISelectionService	selectionService	 = null;
		
		public override ICollection AssociatedComponents {
			get {
				if( base.Control is OutlookBar )
					return ((OutlookBar)base.Control).Tabs;
				else
					return base.AssociatedComponents;
			}
		}
		public IDesignerHost DesignerHost {
			get {
				if( designerHost == null )
					designerHost = (IDesignerHost)GetService( typeof( IDesignerHost ) );
				return designerHost;
			}
		}

		public IToolboxService ToolboxService 	{
			get {
				if( toolboxService == null )
					toolboxService = (IToolboxService)GetService( typeof( IToolboxService ) );
				return toolboxService;
			}
		}

		public ISelectionService SelectionService {
			get {
				if( selectionService == null )
					selectionService = (ISelectionService)GetService( typeof( ISelectionService ) );
				return selectionService;
			}
		}

		protected override bool DrawGrid {
			get { return false; }
		}

		public override bool CanParent( System.Windows.Forms.Control control ) {
			return true;
		}

		protected override void PostFilterProperties( IDictionary Properties ) {

			base.PostFilterProperties( Properties );

			string[] props = { "BackColor", "AutoScroll", 
                               "AutoScrollMargin", "AutoScrollMinSize", 
                               "BackgroundImage" };

			foreach( string property in props )
				Properties.Remove( property );

		}


		protected override void OnDragDrop( DragEventArgs de ) {

			ToolboxItem toolboxItem = this.ToolboxService.DeserializeToolboxItem( de.Data, this.DesignerHost );

			IComponent[] components = toolboxItem.CreateComponents( this.DesignerHost );

			OutlookBar outlookBar = (OutlookBar)this.Control;

			Point pt = outlookBar.PointToClient( new Point( de.X, de.Y ) );

			int TabIndex;
			if( outlookBar.HitTest( HitTestType.CLIENT, pt, out TabIndex ) ) {
				//Replace the current control of the active tab with the newly dropped
				//control
				if( outlookBar.ActiveTab == null ) {
					AddTab( outlookBar, (Control)components[0] );
				} else {
					if( outlookBar.ActiveTab.Child != null ) 
						this.DesignerHost.DestroyComponent( outlookBar.ActiveTab.Child );
					
					Control old = outlookBar.ActiveTab.Child;
					outlookBar.ActiveTab.Child = (Control)components[0];
					RaiseComponentChanged( TypeDescriptor.GetProperties( outlookBar.ActiveTab )["Child"], old, outlookBar.ActiveTab.Child );
				}
			} else {
				AddTab( outlookBar, (Control)components[0] );
			}

			//Set the Selection to the newly dropped control
			this.SelectionService.SetSelectedComponents( components );

			//Inform the Toolbox that the selected it was used, this will reset the mouse to the pointer
			this.ToolboxService.SelectedToolboxItemUsed( );
		}

		protected virtual void AddTab( OutlookBar outlookBar, Control control ) {
			//The control was dropped on the Tabs, so create a new Tab to host the
			//control
			OutlookBar.TabCollection oldTabs = outlookBar.Tabs;
			OutlookBarTab T = (OutlookBarTab)this.DesignerHost.CreateComponent( typeof( OutlookBarTab ) );
			T.Text	= string.Format( "outlookBarTab{0}", oldTabs.Count + 1);
			T.Child = control;
			outlookBar.Tabs.Add( T );

			//Notify the system that the OutlookBar Component is changing.  This is necessary for proper
			//serialization of the Tabs collection
			this.RaiseComponentChanged( TypeDescriptor.GetProperties( outlookBar )["Tabs"], outlookBar.Tabs, oldTabs );
		}
		

		protected override void WndProc( ref Message msg ) {
			base.WndProc( ref msg );
			if( msg.Msg == 0x00000201 ) {	//left mouse down
				OutlookBar outlookBar = this.SelectionService.PrimarySelection as OutlookBar;
				if( outlookBar != null ) {
					int tabIndex;
					if( outlookBar.HitTest( HitTestType.TABS, new Point( msg.LParam.ToInt32( ) ), out tabIndex ) )
						outlookBar.ActiveTabIndex = tabIndex;
				}
			}
		}

		private void OnComponentRemoving( object sender, ComponentEventArgs e ) {
			
			//What is being removed?
			if( e.Component is OutlookBarTab ) {
				OutlookBarTab tab = (OutlookBarTab)e.Component;

			
				//Does the Tab belong to the Current Control?
				if( ((OutlookBar)Control).Tabs.Contains( tab ) ) {
					((OutlookBar)Control).Tabs.Remove( tab );

					if( tab.Child != null )
						DesignerHost.DestroyComponent( tab.Child );
				}
			
			} else if( e.Component is OutlookBar ) {
				//Destroy all tabs
				OutlookBar bar = (OutlookBar)e.Component;
				int tabCount = bar.Tabs.Count;
				for( int index = 0; index < tabCount; index++ )
					DesignerHost.DestroyComponent( bar.Tabs[0] );
				
			} else if( e.Component is Control ) {
				Control control = (Control)e.Component;
				OutlookBar bar = (OutlookBar)Control;
				//Do we own the Control?
				if( bar.Controls.Contains( control ) ) {
					//Find the Tab that owns the control
					foreach( OutlookBarTab tab in bar.Tabs ) {
						if( tab.Child == control ) {
							//Found IT!
							tab.Child = null;
							bar.Controls.Remove( control );
							break;
						}
					}
				}
			}
		}
		
		public override void Initialize( IComponent component ) {
			base.Initialize( component );
			IComponentChangeService changeService = (IComponentChangeService)GetService( typeof(IComponentChangeService) );
			changeService.ComponentRemoving += new ComponentEventHandler( OnComponentRemoving );
		}

		public override void Dispose( ) {
			IComponentChangeService changeService = (IComponentChangeService)GetService( typeof(IComponentChangeService) );
			changeService.ComponentRemoving -= new ComponentEventHandler( OnComponentRemoving );
			base.Dispose( );
		}

		public OutlookBarDesigner() { 			
		}
	}
}
